@@ -1,7 +1,7 @@
 /**
  * \file include/pcm.h
  * \brief Application interface library for the ALSA driver
- * \author Jaroslav Kysela <perex@suse.cz>
+ * \author Jaroslav Kysela <perex@perex.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
  * \author Takashi Iwai <tiwai@suse.de>
  * \date 1998-2001
@@ -362,7 +362,9 @@ enum _snd_pcm_type {
 	SND_PCM_TYPE_IOPLUG,
 	/** External filter plugin */
 	SND_PCM_TYPE_EXTPLUG,
-	SND_PCM_TYPE_LAST = SND_PCM_TYPE_EXTPLUG
+	/** Mmap-emulation plugin */
+	SND_PCM_TYPE_MMAP_EMUL,
+	SND_PCM_TYPE_LAST = SND_PCM_TYPE_MMAP_EMUL
 };
 
 /** PCM type */
