@@ -119,14 +119,14 @@ static void mix_select_callbacks(snd_pcm_direct_t *dmix)
 #else
 
 /* non-concurrent version, supporting both endians */
-static unsigned long long dmix_supported_format =
-	(1ULL << SND_PCM_FORMAT_S16_LE) | (1ULL << SND_PCM_FORMAT_S32_LE) |
-	(1ULL << SND_PCM_FORMAT_S16_BE) | (1ULL << SND_PCM_FORMAT_S32_BE) |
-	(1ULL << SND_PCM_FORMAT_S24_3LE);
+#define generic_dmix_supported_format \
+	((1ULL << SND_PCM_FORMAT_S16_LE) | (1ULL << SND_PCM_FORMAT_S32_LE) |\
+	 (1ULL << SND_PCM_FORMAT_S16_BE) | (1ULL << SND_PCM_FORMAT_S32_BE) |\
+	 (1ULL << SND_PCM_FORMAT_S24_3LE))
 
 #include <byteswap.h>
 
-static void mix_areas1_native(unsigned int size,
+static void generic_mix_areas1_native(unsigned int size,
 			      volatile signed short *dst, signed short *src,
 			      volatile signed int *sum, size_t dst_step,
 			      size_t src_step, size_t sum_step)
@@ -155,7 +155,7 @@ static void mix_areas1_native(unsigned int size,
 	}
 }
 
-static void mix_areas2_native(unsigned int size,
+static void generic_mix_areas2_native(unsigned int size,
 			      volatile signed int *dst, signed int *src,
 			      volatile signed int *sum, size_t dst_step,
 			      size_t src_step, size_t sum_step)
@@ -186,7 +186,7 @@ static void mix_areas2_native(unsigned int size,
 	}
 }
 
-static void mix_areas1_swap(unsigned int size,
+static void generic_mix_areas1_swap(unsigned int size,
 			    volatile signed short *dst, signed short *src,
 			    volatile signed int *sum, size_t dst_step,
 			    size_t src_step, size_t sum_step)
@@ -215,7 +215,7 @@ static void mix_areas1_swap(unsigned int size,
 	}
 }
 
-static void mix_areas2_swap(unsigned int size,
+static void generic_mix_areas2_swap(unsigned int size,
 			    volatile signed int *dst, signed int *src,
 			    volatile signed int *sum, size_t dst_step,
 			    size_t src_step, size_t sum_step)
@@ -247,7 +247,7 @@ static void mix_areas2_swap(unsigned int size,
 }
 
 /* always little endian */
-static void mix_areas3(unsigned int size,
+static void generic_mix_areas3(unsigned int size,
 		       volatile unsigned char *dst, unsigned char *src,
 		       volatile signed int *sum, size_t dst_step,
 		       size_t src_step, size_t sum_step)
@@ -278,16 +278,16 @@ static void mix_areas3(unsigned int size,
 }
 
 
-static void mix_select_callbacks(snd_pcm_direct_t *dmix)
+static void generic_mix_select_callbacks(snd_pcm_direct_t *dmix)
 {
 	if (snd_pcm_format_cpu_endian(dmix->shmptr->s.format)) {
-		dmix->u.dmix.mix_areas1 = mix_areas1_native;
-		dmix->u.dmix.mix_areas2 = mix_areas2_native;
+		dmix->u.dmix.mix_areas1 = generic_mix_areas1_native;
+		dmix->u.dmix.mix_areas2 = generic_mix_areas2_native;
 	} else {
-		dmix->u.dmix.mix_areas1 = mix_areas1_swap;
-		dmix->u.dmix.mix_areas2 = mix_areas2_swap;
+		dmix->u.dmix.mix_areas1 = generic_mix_areas1_swap;
+		dmix->u.dmix.mix_areas2 = generic_mix_areas2_swap;
 	}
-	dmix->u.dmix.mix_areas3 = mix_areas3;
+	dmix->u.dmix.mix_areas3 = generic_mix_areas3;
 }
 
 #endif
